/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005-2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: JoramWriter.java, v 1.1 31 ao�t 07 08:56:47 lam Exp $
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.topology.joram;

import java.io.FileWriter;
import java.io.IOException;

import org.objectweb.petals.topology.TopologyException;
import org.objectweb.petals.topology.generated.Container;
import org.objectweb.petals.topology.generated.Topology;

/**
 * This class is able to create an a3server.xml files for the configuration of
 * Joram service.
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 * @author Roland NAUDIN - eBMWebSourcing
 * @since Petals 2.0
 */
public final class JoramWriter {

    private static final String CONNECTION_MANAGER_CLASS = "org.objectweb.joram.mom.proxies.ConnectionManager";

    private static final String TCP_PROXY_SERVICE_CLASS = "org.objectweb.joram.mom.proxies.tcp.TcpProxyService";

    /**
     * the topology object
     */
    private Topology topology;

    /**
     * the default constructor
     *
     * @param topology
     *            the topology
     */
    public JoramWriter(final Topology topology) {
        this.topology = topology;
    }

    /**
     * write the a3 server file
     *
     * @param fileName
     *            the file name
     * @throws IOException
     *             impossible to create the file
     * @throws TopologyException
     *             impossible to analyze the topology
     */
    public void writeA3Server(final String fileName) throws IOException,
        TopologyException {
        final String buffer = this.generateA3Server();
        final FileWriter writer = new FileWriter(fileName);
        writer.write(buffer);
        writer.close();
    }

    /**
     * Generate the a3 server
     *
     * @return the buffer of the a3server.xml file
     * @throws TopologyException
     *             impossible to analyze the topology
     */
    public String generateA3Server() throws TopologyException {
        String res = null;
        res = this.generateHeader();
        res = res + "<config>\n";
        res = res + "\t<domain name=\"" + this.topology.getDomain().getName()
            + "\" />\n";
        res = res + this.generateServerPart();
        res = res + "</config>\n";
        return res;
    }

    /**
     * generate the header.
     *
     * @return the header
     */
    public String generateHeader() {
        String res = "";
        res = res + "<?xml version=\"1.0\"?>\n";
        res = res + "<!-- <!DOCTYPE config SYSTEM \"a3config.dtd\"> -->\n";
        res = res + "<!--\n";
        res = res + "\tThis is the configuration file for JORAM.\n";
        res = res + "\tIt MUST be used when running PEtALS with the "
            + "static distributed server provided by JORAM.\n";
        res = res + "\n";
        res = res
            + "\tYou MUST dupplicate the values which are defined in the conf/server.properties\n";
        res = res + "\tfile and set them in the current one.\n";
        res = res + "\n";
        res = res
            + "\tMore information on the PEtALS project webpage (http://petals.objectweb.org).\n";
        res = res + "-->\n";
        return res;
    }

    /**
     * Generate the JORAM servers part
     *
     * @return the server part
     */
    public String generateServerPart() {
        String res = "";
        for (final Container container : this.topology.getContainers()
            .getContainer()) {

            res = res + "\t<server id=\""
                + container.getConfiguration().getJoramservice().getId()
                + "\" name=\"" + container.getName()
                + "\" hostname=\"" + container.getHost() + "\">\n";
            res = res
                + "\t\t<network domain=\""
                + this.topology.getDomain().getName()
                + "\" port=\""
                + container.getConfiguration().getJoramservice()
                    .getDomainPort() + "\"/>\n";
            res = res + "\t\t<service class=\"" + CONNECTION_MANAGER_CLASS
                + "\" args=\""
                + container.getConfiguration().getJoramservice().getUser()
                + " "
                + container.getConfiguration().getJoramservice().getPassword()
                + "\"/>\n";
            res = res + "\t\t<service class=\"" + TCP_PROXY_SERVICE_CLASS
                + "\" args=\""
                + container.getConfiguration().getJoramservice().getTcpPort()
                + "\"/>\n";
            res = res + "\t</server>\n";
        }

        return res;
    }

}
